"use strict";

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _require = require("react"),
    useState = _require.useState,
    useEffect = _require.useEffect,
    useRef = _require.useRef;

var initialState = {
  lastMessage: undefined,
  messages: []
};

var supportsBroadcastAPI = function () {
  return window && window.BroadcastChannel;
}();

function useTabCommunication(channel_name) {
  var _useState = useState(initialState),
      _useState2 = _slicedToArray(_useState, 2),
      state = _useState2[0],
      setMessages = _useState2[1];

  var channel = void 0;

  if (supportsBroadcastAPI) {
    channel = useRef(new BroadcastChannel(channel_name));
  }

  var postMessage = function postMessage(message) {
    if (message) {
      var msg = JSON.stringify({
        message: message,
        time: Date.now()
      });

      if (supportsBroadcastAPI && channel) {
        channel.current.postMessage(msg);
      } else {
        window.localStorage.setItem(channel_name, msg);
      }
    }
  };

  var updateState = function updateState(data) {
    setMessages(function (state) {
      return {
        lastMessage: data.message,
        messages: state.messages.concat(data.message)
      };
    });
  };

  var updateFromLocalStorage = function updateFromLocalStorage(e) {
    var data = JSON.parse(e.newValue);
    updateState(data);
  };

  useEffect(function () {
    if (supportsBroadcastAPI) {
      if (channel && channel.current) {
        channel.current.onmessage = function (e) {
          return updateState(JSON.parse(e.data));
        };
      }
    } else {
      window.addEventListener("storage", updateFromLocalStorage);
    }

    return function cleanup() {
      if (channel.current) {
        channel.close();
      } else {
        window.localStorage.removeItem(channel_name);
        window.removeEventListener("storage", updateFromLocalStorage);
      }
    };
  }, [channel_name]);

  return [state, postMessage];
}

module.exports = useTabCommunication;